package com.bull.eclipse.jonas;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.IDebugEventSetListener;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import com.bull.eclipse.jonas.actions.StartActionDelegate;
import com.bull.eclipse.jonas.editors.ProjectListElement;


/**
 * The main plugin class to be used in the desktop.
 */
public class JonasLauncherPlugin extends AbstractUIPlugin implements IDebugEventSetListener {

	public static final String PLUGIN_ID = "com.bull.eclipse.jonas" ; 
	public static final String NATURE_ID = PLUGIN_ID + ".jonasnature" ; 

	public static final String JONAS_PREF_HOME_KEY = "jonasDir";
	public static final String JONAS_PREF_BASE_KEY = "jonasBase";
	public static final String JONAS_PREF_VERSION_KEY = "jonasVersion";
	public static final String JONAS_PREF_JRE_KEY = "jonasJRE";
	public static final String JONAS_PREF_JVM_PARAMETERS_KEY = "jvmParameters";
	public static final String JONAS_PREF_JVM_CLASSPATH_KEY = "jvmClasspath";
	public static final String JONAS_PREF_JVM_BOOTCLASSPATH_KEY = "jvmBootClasspath";
	public static final String JONAS_PREF_PROJECTSINCP_KEY = "projectsInCp";
	public static final String JONAS_PREF_XTRA_CLASSPATH_KEY = "xtraClasspath";
	public static final String JONAS_EXTRA_RESOURCES_KEY = "extraResources";
	public static final String JONAS_PREF_ORB_KEY = "orb";
	public static final String JONAS_PREF_DEBUGMODE_KEY = "jonasDebugMode";
	public static final String JONAS_PREF_CONSOLEIN_KEY = "jonasConsoleIn";
	public static final String JONAS_VERSION30 = "jonas30";
	public static final String JONAS_VERSION31 = "jonas31";
	public static final String JONAS_VERSION40 = "jonas40";
	public static final String JONAS_VERSION41 = "jonas41";
	public static final String JONAS_ORB_RMI = "RMI";
	public static final String JONAS_ORB_JEREMIE = "JEREMIE";
	public static final String JONAS_PREF_DATABASE_KEY = "database";
	public static final String JONAS_PREF_AUTOMATICDEPLOY_KEY = "automaticDeploy";
				
	private static final String JONAS_HOME_CLASSPATH_VARIABLE = "JONAS_ROOT";


	//The shared instance.
	private static JonasLauncherPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	// Restart management
	private boolean eventRestart = false;
	
	/**
	 * The constructor.
	 */
	public JonasLauncherPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;

		try {
			resourceBundle= PropertyResourceBundle.getBundle("resources");
		} catch (MissingResourceException x) {
			JonasLauncherPlugin.log("Problme lors du chargement du plugin");
			x.printStackTrace();
			resourceBundle = null;
		}
		
		JonasLauncherPlugin.getWorkspace().addResourceChangeListener(new JonasProjectChangeListener(), IResourceChangeEvent.PRE_DELETE);
	}

	/**
	 * Returns the shared instance.
	 */
	public static JonasLauncherPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= JonasLauncherPlugin.getDefault().getResourceBundle();
		System.out.println("bundle = " + bundle);
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		try {
			resourceBundle= PropertyResourceBundle.getBundle("resources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
		return resourceBundle;
	}


	public String getJonasDir() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getString(JONAS_PREF_HOME_KEY);
	}

	public String getBaseDir() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getString(JONAS_PREF_BASE_KEY);
	}
	
	public String getJonasVersion() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		String result = pref.getString(JONAS_PREF_VERSION_KEY);
		if (result.equals("")) 
			result = JONAS_VERSION30;
		
		return result;
	}

	public String getJonasJRE() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		String result = pref.getString(JONAS_PREF_JRE_KEY);
		if (result.equals("")) 
			result = JavaRuntime.getDefaultVMInstall().getId();
			
		return result;
	}
	
	public boolean isDebugMode() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return !pref.getBoolean(JONAS_PREF_DEBUGMODE_KEY);
	}

	public boolean isConsoleIn() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getBoolean(JONAS_PREF_CONSOLEIN_KEY);
	}

	public String getJvmParamaters() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getString(JONAS_PREF_JVM_PARAMETERS_KEY);		
	}

	public String getJvmClasspath() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getString(JONAS_PREF_JVM_CLASSPATH_KEY);		
	}

	public String getJvmBootClasspath() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getString(JONAS_PREF_JVM_BOOTCLASSPATH_KEY);		
	}

	public String getDatabase() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getString(JONAS_PREF_DATABASE_KEY);		
	}

	public String getXtraClasspath() {
			IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
			return pref.getString(JONAS_PREF_XTRA_CLASSPATH_KEY);		
	}

	public static String getPluginId()
	{
		return getDefault().getDescriptor().getUniqueIdentifier();
	}

	public String getOrb() {
			IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
			return pref.getString(JONAS_PREF_ORB_KEY);		
	}
	
	public String getAutomaticDeploy() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		return pref.getString(JONAS_PREF_AUTOMATICDEPLOY_KEY);				
	}
			
	public void setProjectsInCP(List projectsInCP) {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		StringBuffer buf = new StringBuffer();
		Iterator it = projectsInCP.iterator();
		while(it.hasNext()) {
			ProjectListElement each = (ProjectListElement)it.next();
			buf.append(each.getID());
			buf.append(';');	
		}
		pref.setValue(JONAS_PREF_PROJECTSINCP_KEY, buf.toString());
	}

	public List getProjectsInCP() {
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		String stringList =  pref.getString(JONAS_PREF_PROJECTSINCP_KEY);

		List projectsIdList = new ArrayList();	
		StringTokenizer tokenizer = new StringTokenizer(stringList, ";");
		while (tokenizer.hasMoreElements()) {
			projectsIdList.add(tokenizer.nextToken());
		}
		
		return ProjectListElement.stringsToProjectsList(projectsIdList);
		
	}
	
		
	public JonasBootstrap getJonasBootstrap() {
		JonasBootstrap jonasBootsrap = null;

		if(getJonasVersion().equals(JONAS_VERSION30)) {
			jonasBootsrap = new Jonas30Bootstrap();
		}
		
		if(getJonasVersion().equals(JONAS_VERSION31)) {
					jonasBootsrap = new Jonas31Bootstrap();
		}

		if(getJonasVersion().equals(JONAS_VERSION40)) {
			jonasBootsrap = new Jonas40Bootstrap();
		}

		if(getJonasVersion().equals(JONAS_VERSION41)) {
			jonasBootsrap = new Jonas41Bootstrap();
		}

		return jonasBootsrap;
	}

		

	static public void log(String msg) {
		ILog log = JonasLauncherPlugin.getDefault().getLog();
		Status status = new Status(IStatus.ERROR, JonasLauncherPlugin.getDefault().getDescriptor().getUniqueIdentifier(), IStatus.ERROR, msg + "\n", null);
		log.log(status);
	}
	
	static public void log(Exception ex) {
		ILog log = JonasLauncherPlugin.getDefault().getLog();
		StringWriter stringWriter = new StringWriter();
	    ex.printStackTrace(new PrintWriter(stringWriter));
		String msg = stringWriter.getBuffer().toString();

		Status status = new Status(IStatus.ERROR, JonasLauncherPlugin.getDefault().getDescriptor().getUniqueIdentifier(), IStatus.ERROR, msg, null);
		log.log(status);
	}

	
	public IPath getJonasIPath() {
		IPath jonasPath = getJonasClasspathVariable();
		if(jonasPath == null) {
			return new Path(JonasLauncherPlugin.getDefault().getJonasDir());
		} else {
			return new Path(JONAS_HOME_CLASSPATH_VARIABLE);
		}	
	}
	
	private IPath getJonasClasspathVariable() {
		IPath jonasPath = JavaCore.getClasspathVariable(JONAS_HOME_CLASSPATH_VARIABLE);
		if(jonasPath == null) {
			this.initJonasClasspathVariable();
			jonasPath = JavaCore.getClasspathVariable(JONAS_HOME_CLASSPATH_VARIABLE);
		}
		return jonasPath;
	}

	public void initJonasClasspathVariable() {
		try {
			JavaCore.setClasspathVariable(
				JONAS_HOME_CLASSPATH_VARIABLE,
				new Path(JonasLauncherPlugin.getDefault().getJonasDir()),
				null);		
		} catch (JavaModelException e) {
			log(e);
		}		
	}



	// Restart managed with event listener. 

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugin#startup()
	 */
	public void startup() throws CoreException {
		super.startup();
		DebugPlugin.getDefault().addDebugEventListener(this);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.core.IDebugEventSetListener#handleDebugEvents(org.eclipse.debug.core.DebugEvent[])
	 */
	public void handleDebugEvents(DebugEvent[] events)
	{
		for (int i = 0; i < events.length; i++) {
			if (events[i].getKind() == DebugEvent.CREATE) {
				if (events[i].getSource() instanceof IProcess) {
					IProcess process = (IProcess) events[i].getSource();
				}
			}
			else if (events[i].getKind() == DebugEvent.TERMINATE) {
					if (events[i].getSource() instanceof IProcess) {
						IProcess process = (IProcess) events[i].getSource();
						if(eventRestart == true) {
							eventRestart = false;
							StartActionDelegate start = new StartActionDelegate();
							start.run(start.getAction());
						}
					}
				}
			}
		}


	public boolean isEventRestart() {
		return eventRestart;
	}

	public void setEventRestart(boolean b) {
		eventRestart = b;
	}

	
}

