 package com.bull.eclipse.jonas;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */
 
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;

import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.JDTUtil;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;




public class JonasProject extends PlatformObject implements IProjectNature  {

	// Persistence properties of projects
	private static final String PROPERTIES_FILENAME = ".jonasplugin";
	private static final String KEY_WEBCONTEXT = "webContext";
	private static final String KEY_EXPORTSOURCE = "exportSource";
	private static final String KEY_RELOADABLE="reloadable";
	private static final String KEY_REDIRECTLOGGER="redirectLogger";	
	private static final String KEY_WARLOCATION = "warLocation";
	private static final String KEY_SRCDIR  = "srcDir";
	private static final String KEY_SRCEJBDIR  = "srcEJBDir";
	private static final String KEY_WEBDIRNAME  = "webdirName";
	private static final String KEY_EXTRARESOURCES = ExtraResourcesEntries.TAG_NAME;

	private static final QualifiedName QN_WEBCONTEXT = new QualifiedName("JonasProject", KEY_WEBCONTEXT);
	private static final QualifiedName QN_EXPORTSOURCE = new QualifiedName("JonasProject", KEY_EXPORTSOURCE);
	private static final QualifiedName QN_RELOADABLE = new QualifiedName("JonasProject", KEY_RELOADABLE);
	private static final QualifiedName QN_REDIRECTLOGGER = new QualifiedName("JonasProject", KEY_REDIRECTLOGGER);
	private static final QualifiedName QN_WARLOCATION = new QualifiedName("JonasProject", KEY_WARLOCATION);
	private static final QualifiedName QN_SRCDIR = new QualifiedName("JonasProject", KEY_SRCDIR);
	private static final QualifiedName QN_SRCEJBDIR = new QualifiedName("JonasProject", KEY_SRCEJBDIR);
	private static final QualifiedName QN_WEBDIRNAME = new QualifiedName("JonasProject", KEY_WEBDIRNAME);
	private static final QualifiedName QN_EXTRARESOURCES = new QualifiedName("JonasProject", KEY_EXTRARESOURCES);
	
	/**
	 * The platform project this <code>JonasProject</code> is based on
	 */
	protected IProject project;
	protected IJavaProject javaProject;

	protected String webContext = "";
	protected String warLocation = "";
	protected boolean exportSource;
	protected boolean reloadable = true;
	protected boolean redirectLogger = true;
	protected String srcDir = "";
	protected String srcEJBDir = "";
	protected String webdirName = "";
	protected ExtraResourcesEntries extraResourcesEntries;
	protected Vector packageEJB = null;
	protected Vector jarEJBName = null;
	
	protected IFolder webDirFolder;
	protected IFolder srcDirFolder;
	protected IFolder srcEJBDirFolder;
	protected boolean createSamples; // value from the proj creation wizard

	/**
	 * Gets the project.
	 * @return Returns a IProject
	 */
	public IProject getProject() {
		return project;
	}

	/**
	 * Sets the project.
	 * @param project The project to set
	 */
	public void setProject(IProject project) {
		this.project = project;
	}
	
	/*
	 * @see IProjectNature#configure()
	 */
	public void configure() throws CoreException {
	}

	/*
	 * @see IProjectNature#deconfigure()
	 */
	public void deconfigure() throws CoreException {
	}

	/*
	 * @see IProjectNature#getProject()
	 */
	public IJavaProject getJavaProject() {
		return javaProject;
	}

	/*
	 * @see IProjectNature#setProject(IProject)
	 */
	public void setJavaProject(IJavaProject javaProject) {
		this.javaProject = javaProject;
		this.setProject(javaProject.getProject());
	}


	static public void addJonasNature(IJavaProject project) {
		try {
			JDTUtil.addNatureToProject(project.getProject(), JonasLauncherPlugin.NATURE_ID); 	
		} catch(CoreException ex) {
			JonasLauncherPlugin.log(ex.getMessage());
		}
	}

	static public void removeJonasNature(IJavaProject project) {
		try {
			JDTUtil.removeNatureToProject(project.getProject(), JonasLauncherPlugin.NATURE_ID); 	
		} catch(CoreException ex) {
			JonasLauncherPlugin.log(ex.getMessage());
		}
	}

	/**
	 * Return a JonasProject if this javaProject has the jonas nature
	 * Return null if Project has not jonas nature
	 */
	static public JonasProject create(IJavaProject javaProject) {
		JonasProject result = null;
		try {
			result = (JonasProject)javaProject.getProject().getNature(JonasLauncherPlugin.NATURE_ID);
			if(result != null)
				result.setJavaProject(javaProject);
		} catch(CoreException ex) {
			JonasLauncherPlugin.log(ex.getMessage());
		}
		return result;
	}

	/**
	 * Return a JonasProject if this Project has the jonas nature
	 * Return null if Project has not jonas nature
	 */
	static public JonasProject create(IProject project) {

		IJavaProject javaProject = JavaCore.create(project);
		if(javaProject != null) {
			return JonasProject.create(javaProject);
		} else {
			return null;
		}
	}	

	private File getPropertiesFile() {
		return (this.getProject().getLocation().append(PROPERTIES_FILENAME).toFile());
	}
	
	private String readProperty(String key) {
		String result = null;
		
		try {
			result = FileUtil.readPropertyInXMLFile(getPropertiesFile(), key);
		} catch (IOException e) {
			try {
				result = getJavaProject().getCorrespondingResource().getPersistentProperty(new QualifiedName("JonasProject", key));
			} catch (Exception e2) {
				JonasLauncherPlugin.log(e2);
			}
		}
		
		if(result == null) {
			result = "";
		}
			
		return result;
	}

		
	/**
	 * Gets the webContext.
	 * @return Returns a String
	 */
	public String getWebContext() {
		return this.readProperty(KEY_WEBCONTEXT);
	}	

	/**
	 * Sets the webContext.
	 * @param webContext The webContext to set
	 */
	public void setWebContext(String wp) {
		this.webContext = wp;
	}

	public void updateWebContext(String newContextPath) throws Exception {
		setWebContext(newContextPath);	
	}
		
	/**
	 * Gets the warfile.
	 * @return Returns a String
	 */
	public String getWarLocation() {
		return this.readProperty(KEY_WARLOCATION);
	}

	/**
	 * Sets the warfile
	 * @param warfile The warfile to set
	 */
	public void setWarLocation(String wl) {
		this.warLocation = wl;
	}
	

	/**
	 * Gets the exportSource.
	 * @return Returns a boolean
	 */
	public boolean getExportSource() {
		return new Boolean(this.readProperty(KEY_EXPORTSOURCE)).booleanValue();
	}

	/**
	 * Sets the exportSource.
	 * @param exportSource The exportSource to set
	 */
	public void setExportSource(boolean exportSource) {
		this.exportSource = exportSource;
	}

	/**
	 * Gets the reloadable
	 * @return Returns a boolean
	 */
	public boolean getReloadable(){
		String reloadableProperty = this.readProperty(KEY_RELOADABLE);
		// Set default value to true
		if(reloadableProperty.equals("")) {
			reloadableProperty = "true";	
		}
		return new Boolean(reloadableProperty).booleanValue();
	}
	
	/**
	 * Sets the reloadable
	 * @param reloadable The reloadable to set
	 */
	public void setReloadable(boolean reloadable){
		this.reloadable = reloadable;
	}

	/**
	 * Gets the redirectLogger
	 * @return Returns a boolean
	 */
	public boolean getRedirectLogger(){
		String redirectLoggerProperty = this.readProperty(KEY_REDIRECTLOGGER);
		// Set default value to false
		if(redirectLoggerProperty.equals("")) {
			redirectLoggerProperty = "false";	
		}
		return new Boolean(redirectLoggerProperty).booleanValue();
	}
	
	/**
	 * Sets the redirectLogger
	 * @param redirectLogger The redirectLogger to set
	 */
	public void setRedirectLogger(boolean redirectLogger){
		this.redirectLogger = redirectLogger;
	}
	


	/**
	 * Gets the srcdir
	 * @return Returns a String
	 */
	public String getSrcDir(){
		return this.readProperty(KEY_SRCDIR);
	}

	public String getSrcEJBDir(){
		return this.readProperty(KEY_SRCEJBDIR);
	}

	/**
	 * Sets the srcdir
	 * @param srcdir The srcdir to set
	 */
	public void setSrcDir(String srcdir){
		this.srcDir = srcdir;
	}

	
	public void setSrcEJBDir(String srcejbdir){
		this.srcEJBDir = srcejbdir;
	}

	/**
	 * Gets the webdirName
	 * @return Returns a String
	 */
	public String getWebdirName(){
		return this.readProperty(KEY_WEBDIRNAME);
	}
	
	/**
	 * Sets the webdirName
	 * @param webdirName The webdirName to set
	 */
	public void setWebdirName(String webdirName){
		this.webdirName = webdirName;
	}


	/**
	 * set the extra resources entries which shall be loaded 
	 * @param entries List of Extra Resources Entry objects
	 */
	public void setExtraResourcesEntries(ExtraResourcesEntries entries) {
		extraResourcesEntries = entries;
	}
	
	/** 
	 * return the webclasspath entries
	 */
	public ExtraResourcesEntries getExtraResourcesEntries() {
		try {
			return ExtraResourcesEntries.xmlUnmarshal(FileUtil.readTextFile(getPropertiesFile()));
		} catch(IOException ioEx) {
			return null;
		}
	}


	/*
	 * Store exportSource in project persistent properties
	 */
	public void saveProperties() {
		try {
			StringBuffer fileContent = new StringBuffer();
			fileContent.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			fileContent.append("<jonasProjectProperties>\n");
			fileContent.append("    <exportSource>" + exportSource + "</exportSource>\n");
			fileContent.append("    <reloadable>" + reloadable + "</reloadable>\n");
			fileContent.append("    <redirectLogger>" + redirectLogger + "</redirectLogger>\n");
			fileContent.append("    <warLocation>" + warLocation + "</warLocation>\n");
			fileContent.append("    <srcDir>" + srcDir + "</srcDir>\n");
			fileContent.append("    <srcEJBDir>" + srcEJBDir + "</srcEJBDir>\n");
			fileContent.append("    <webdirName>" + webdirName + "</webdirName>\n");			
			fileContent.append("    <webContext>" + webContext + "</webContext>\n");
			if (packageEJB != null) {
				int i = 1;
				for (Enumeration e = packageEJB.elements() ; e.hasMoreElements() ;) {
					fileContent.append("    <ejb" + i + ">" + e.nextElement() + "</ejb" + i + ">\n");
					i++;			
				}
			}
			if (jarEJBName != null) {
				int i = 1;
				for (Enumeration e = jarEJBName.elements() ; e.hasMoreElements() ;) {
					fileContent.append("    <ejbjarname" + i + ">" + e.nextElement() + "</ejbjarname" + i + ">\n");
					i++;			
				}
			}

			if (extraResourcesEntries != null) {
				fileContent.append(extraResourcesEntries.xmlMarshal(4));
			}
			fileContent.append("</jonasProjectProperties>\n");
			FileUtil.toTextFile(getPropertiesFile(), fileContent.toString());			
		} catch (Exception ex) {
			JonasLauncherPlugin.log(ex.getMessage());	
		}			
	}

	public void saveEJBProperties() {
		try {
			StringBuffer fileContent = new StringBuffer();
			fileContent.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			fileContent.append("<jonasProjectProperties>\n");
			fileContent.append("    <exportSource>" + getExportSource() + "</exportSource>\n");
			fileContent.append("    <reloadable>" + getReloadable() + "</reloadable>\n");
			fileContent.append("    <redirectLogger>" + getRedirectLogger() + "</redirectLogger>\n");
			fileContent.append("    <warLocation>" + getWarLocation() + "</warLocation>\n");
			fileContent.append("    <srcDir>" + getSrcDir() + "</srcDir>\n");
			fileContent.append("    <srcEJBDir>" + getSrcEJBDir() + "</srcEJBDir>\n");
			fileContent.append("    <webdirName>" + getWebdirName() + "</webdirName>\n");			
			fileContent.append("    <webContext>" + getWebContext() + "</webContext>\n");
			if (packageEJB != null) {
				int i = 1;
				for (Enumeration e = packageEJB.elements() ; e.hasMoreElements() ;) {
					fileContent.append("    <ejb" + i + ">" + e.nextElement() + "</ejb" + i + ">\n");
					i++;			
				}
			}
			if (jarEJBName != null) {
				int i = 1;
				for (Enumeration e = jarEJBName.elements() ; e.hasMoreElements() ;) {
					fileContent.append("    <ejbjarname" + i + ">" + e.nextElement() + "</ejbjarname" + i + ">\n");
					i++;			
				}
			}

			if (getExtraResourcesEntries() != null) {
				fileContent.append(getExtraResourcesEntries().xmlMarshal(4));
			}
			fileContent.append("</jonasProjectProperties>\n");
			FileUtil.toTextFile(getPropertiesFile(), fileContent.toString());			
		} catch (Exception ex) {
			JonasLauncherPlugin.log(ex.getMessage());	
		}			
	}

	/**
	 * Run all the steps to configure a JavaProject as a JonasProject
	 */
	public void fullConfiguration() throws CoreException, IOException {

		this.createWEBINFFolder();
		this.createSrcFolder();

		this.createWebSample();

		this.addJonasJarToProjectClasspath();
		this.addWEBINFLibJarFilesToProjectClasspath();

		this.clearDefaultSourceEntries();
		this.setClassesAsOutputFolder();
		if(classesContainsJavaFiles()) {
			this.setClassesAsSourceFolder();
		}
		this.setSrcAsSourceFolder(srcDir);

	}

	public void clearDefaultSourceEntries()  throws CoreException {

		IClasspathEntry[] entries = javaProject.getRawClasspath();
		List cp= new ArrayList(entries.length + 1);
		for(int i=0; i<entries.length; i++) {
			if(entries[i].getEntryKind() != IClasspathEntry.CPE_SOURCE) {
				cp.add(entries[i]);
			}
		}
		javaProject.setRawClasspath((IClasspathEntry[])cp.toArray(new IClasspathEntry[cp.size()]), null);
	}

	/*
	 * Add Jonas_RMI.jar, Jonas_JEREMIE.jar and web_catalina.jar to project classpath
	 */	
	public void addJonasJarToProjectClasspath()  throws CoreException {

		Vector jonasJarPath = JonasLauncherPlugin.getDefault().getJonasBootstrap().getJonasJarPath();
		Vector jonasWebJarPath = JonasLauncherPlugin.getDefault().getJonasBootstrap().getJonasWebJarPath();


		IClasspathEntry[] entries = javaProject.getRawClasspath();
		List cp= new ArrayList(entries.length + 1);
		cp.addAll(Arrays.asList(entries));

		if(jonasJarPath != null) {
			for(int i = 0; i < jonasJarPath.size();i++) {
				cp.add(JavaCore.newVariableEntry(JonasLauncherPlugin.getDefault().getJonasIPath().append((IPath) jonasJarPath.elementAt(i)), null, null));				
			}
		}
		if(jonasWebJarPath != null)	{		
			for(int j = 0; j < jonasWebJarPath.size();j++) {
				cp.add(JavaCore.newVariableEntry(JonasLauncherPlugin.getDefault().getJonasIPath().append((IPath) jonasWebJarPath.elementAt(j)), null, null));
			}
		}


		javaProject.setRawClasspath((IClasspathEntry[])cp.toArray(new IClasspathEntry[cp.size()]), null);
	}
	
	public void removeJonasJarToProjectClasspath()  throws CoreException {

		IClasspathEntry[] entries = javaProject.getRawClasspath();
		List cp = new ArrayList(entries.length + 1);
		cp.addAll(Arrays.asList(entries));
		Iterator cpIte = cp.iterator();
		List newCp = new ArrayList();
		
		while (cpIte.hasNext()) {
			IClasspathEntry pathEntry = (IClasspathEntry) cpIte.next();
			String lastSeg = pathEntry.getPath().lastSegment();
			JonasLauncherPlugin.log("Last seg = " + lastSeg);
			if (lastSeg.equals("jonas.jar") || 
				lastSeg.equals("web_catalina.jar") ||
				lastSeg.equals("mail.jar") ||
				lastSeg.equals("jts-spec1_0.jar") ||
				lastSeg.equals("jta-spec1_0_1.jar") ||
				lastSeg.equals("jms.jar") ||
				lastSeg.equals("servlet-2_4.jar") ||
				lastSeg.equals("jsp-2_0.jar") ||
				lastSeg.equals("jacc-1_0.jar") ||
				lastSeg.equals("javax77.jar") ||
				lastSeg.equals("ejb-2_1-api.jar") ||
				lastSeg.equals("connector-1_5.jar") ||
				lastSeg.equals("activation.jar") ||
				lastSeg.equals("RMI_jonas.jar") ||
				lastSeg.equals("JEREMIE_jonas.jar"))
				continue;
			else
				newCp.add(pathEntry);
			
		}
		javaProject.setRawClasspath((IClasspathEntry[])newCp.toArray(new IClasspathEntry[newCp.size()]), null);
	}

	
	/*
	 * Add all jar in WEB-INF/lib to project classpath
	 */
	public void addWEBINFLibJarFilesToProjectClasspath()  throws CoreException {
		IFolder libFolder = this.getWebInfFolder().getFolder("lib");
		IResource[] libFiles = libFolder.members();

		IClasspathEntry[] entries = javaProject.getRawClasspath();
		List cp= new ArrayList(entries.length + 1);
		cp.addAll(Arrays.asList(entries));
		
		for(int i=0; i<libFiles.length; i++) {
			if((libFiles[i].getType() == IResource.FILE) && (libFiles[i].getFileExtension().equalsIgnoreCase("jar"))) {
				cp.add(JavaCore.newLibraryEntry(libFiles[i].getFullPath(), null, null));
			}				
		}

		javaProject.setRawClasspath((IClasspathEntry[])cp.toArray(new IClasspathEntry[cp.size()]), null);
	}
	
	public IFolder getWebInfFolder() {		
		if (getWebDirFolder() == null) {
			return project.getFolder("WEB-INF");
		} else {						
			return getWebDirFolder().getFolder("WEB-INF");
		}
	}


	//public IFolder getProjectFolder() {
	//	return new IFolder()
	//}
	public IFolder getWebDirFolder() {
		if (webDirFolder == null) {			
			this.initWebDirFolder(false);
		}				
		return webDirFolder;
	}
	
	public IFolder getSrcDirFolder() {
		if (srcDirFolder == null) {
			this.initSrcDirFolder(false);
		}
		return srcDirFolder;
	}		

	public IFolder getSrcEJBDirFolder() {
		if (srcEJBDirFolder == null) {
			this.initSrcEJBDirFolder(false);
		}
		return srcEJBDirFolder;
	}		


	
	private IFolder initWebDirFolder(boolean create) {

		StringTokenizer tokenizer = new StringTokenizer(this.getWebdirName(), "/\\:");
		IFolder folder = null;
		try {
			while (tokenizer.hasMoreTokens()) {				
				String each = tokenizer.nextToken();				
				if (folder == null) {
					folder = project.getFolder(each);
				} else {
					folder = folder.getFolder(each);
				}			
				if (create) {					
					this.createFolder(folder);
				}
			}
		} catch (CoreException ex) {
			JonasLauncherPlugin.log(ex);
			folder = null;
			setWebdirName("/");
		}
		this.webDirFolder = folder;		
		return folder;
	}

	private IFolder initSrcDirFolder(boolean create) {

		StringTokenizer tokenizer = new StringTokenizer(this.getSrcDir(), "/\\:");
		IFolder folder = null;
		try {
			while (tokenizer.hasMoreTokens()) {
				String each = tokenizer.nextToken();
				if (folder == null) {
					folder = project.getFolder(each);
				} else {
					folder = folder.getFolder(each);
				}
				if (create) {
					this.createFolder(folder);
				}
			}
		} catch (CoreException ex) {
			JonasLauncherPlugin.log(ex);
			folder = null;			
		}
		this.srcDirFolder = folder;
		return folder;
	}

	private IFolder initSrcEJBDirFolder(boolean create) {
		StringTokenizer tokenizer = new StringTokenizer(this.getSrcEJBDir(), "/\\:");
		IFolder folder = null;
		try {
			while (tokenizer.hasMoreTokens()) {
				String each = tokenizer.nextToken();
				if (folder == null) {
					folder = project.getFolder(each);
				} else {
					folder = folder.getFolder(each);
				}
				if (create) {
					this.createFolder(folder);
				}
			}
		} catch (CoreException ex) {
			JonasLauncherPlugin.log(ex);
			folder = null;
		}
		this.srcEJBDirFolder = folder;
		return folder;
	}

	public void createWEBINFFolder()  throws CoreException {
		IFolder webinfFolder = this.getWebInfFolder();
		if ((webDirFolder != null) && (!(webDirFolder.getName().trim()).equals(".")))
			this.createFolder(webDirFolder);			
		this.createFolder(webinfFolder);
		this.createFolder(webinfFolder.getFolder("classes"));		
		this.createFolder(webinfFolder.getFolder("lib"));
	}

	public void createSrcFolder()  throws CoreException {

		//this.createFolder(this.getWebInfFolder().getFolder("src"));
		IFolder srcdirFolder = this.getSrcDirFolder();
		if (srcDirFolder != null)
			this.createFolder(srcdirFolder);
	}
	
	/**
	 * create sample html and java src files.
	 * copy the skeletons from the template directory,
	 * editing them as necessary.
	 * @author steve krupanszky, august 2003
	 * 
	 * NOTE: The creation of these sample files should be
	 * based on "Create a Web Application" and/or
	 * "Create an EJB" being set, too.
	*/
	public void createWebSample()
	{
		String template = null;
		String webxmlTemplate = null;
		IFile webxml = null;
		if( ! createSamples ) { 
			template = TemplateDirUtil.getTemplateDir();
			webxmlTemplate = template + "/web.xml.empty";
			webxml = getWebInfFolder().getFile("web.xml");
			createFile( webxmlTemplate, webxml );		
			return; 
		}

		template = TemplateDirUtil.getTemplateDir();
		webxmlTemplate = template + "/web.xml";
		webxml = getWebInfFolder().getFile("web.xml");
		createFile( webxmlTemplate, webxml );

		String indexhtmlTemplate = template + "/index.html";
		IFile indexhtml = getWebDirFolder().getFile("index.html");
		createFile( indexhtmlTemplate, indexhtml );

		String samplejspTemplate = template + "/sample-jsp.jsp";
		IFile samplejsp = getWebDirFolder().getFile("sample-jsp.jsp");
		createFile( samplejspTemplate, samplejsp );

		String sampleservletTemplate = template + "/SampleServlet-src";
		IFile sampleservlet = getSrcDirFolder().getFile("SampleServlet.java");
		createFile( sampleservletTemplate, sampleservlet );
		
		// and create a skeleton jonas-web.xml file
		try {
			StringBuffer fileContent = new StringBuffer();
			fileContent.append(
				"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"
				+ "<!DOCTYPE jonas-web-app PUBLIC\n"
				+ " \"-//ObjectWeb//DTD JOnAS Web App 2.6//EN\"\n"
				+ " \"http://www.objectweb.org/jonas/dtds/jonas-web-app_2_6.dtd\">\n"
				+ "\n"
				+ "<jonas-web-app>\n"
				+ "\t<context-root>" + webContext + "</context-root>\n"
				+ "</jonas-web-app>\n"
			);
			IFile jonaswebxml = getWebInfFolder().getFile("jonas-web.xml");
			JonasLauncherPlugin.log("JOPE: creating "  + jonaswebxml.getFullPath().toString() );
			jonaswebxml.create(new java.io.ByteArrayInputStream(fileContent.toString().getBytes()),false,null);
		}
		catch( Exception e ) {
			JonasLauncherPlugin.log("JOPE: exception creating file: " + e );
		}
	}
	
	
	public void createFile( String from, IFile to )
	{
		JonasLauncherPlugin.log("JOPE: copying " + from + " to " + to.getFullPath().toString() );
		try {
			InputStream stream = new FileInputStream(from);
			to.create( stream, false, null );
		}
		catch( Exception e ) {
			JonasLauncherPlugin.log("JOPE: exception copying file: " + e );
		}
	}


	public Vector getPackageProjectFolder() {
		readEJBPkgName();
		return packageEJB;
	}

	private void readEJBPkgName() {
		if ((packageEJB == null) || (jarEJBName == null)) {
			int i = 1;
			boolean isEJB = true;
			String packEJB = null;
			String jarName = null;
			while (isEJB) {
				packEJB = this.readProperty("ejb" + i);
				jarName = this.readProperty("ejbjarname" + i);
				if (packEJB.equals("")) {
					isEJB = false;
				} else {
					if ((new File(this.getProject().getLocation().append(getSrcEJBDir()).append(packEJB).toOSString()).exists())) {
						if (packageEJB == null) 
							packageEJB = new Vector();
						if (jarEJBName  == null)	
							jarEJBName = new Vector();
						packageEJB.add(packEJB);
						jarEJBName.add(jarName);
					}
				}
				i++;
			}			 
		} else {
			for (Enumeration e = packageEJB.elements() ; e.hasMoreElements() ;) {
				String pkgEJB = (String)e.nextElement();
				if (!(new File(this.getProject().getLocation().append(getSrcEJBDir()).append(pkgEJB).toOSString()).exists())) {
					int i = packageEJB.indexOf(pkgEJB);
					packageEJB.remove(i);
					jarEJBName.remove(i);	
				}
			}
		}
		saveEJBProperties();
	}
	public Vector getJarEjbName() {
		readEJBPkgName();
		return jarEJBName;
	}

	public void setPackageProjectFolder(String packageDir) {
		if (packageEJB == null) 
			packageEJB = new Vector();
		packageDir = packageDir.replace(".".charAt(0),File.separatorChar);	
		JonasLauncherPlugin.log("package = " + packageDir);
		packageEJB.add(packageDir);
		saveEJBProperties();
	}

	public void setJarEjbName(String jarejbName) {
		if (jarEJBName == null) 
			jarEJBName = new Vector();
		JonasLauncherPlugin.log("jarEJBName = " + jarejbName);
		jarEJBName.add(jarejbName);
		saveEJBProperties();
	}

	public void setSrcAsSourceFolder(String srcFolder)  throws CoreException {
		this.setFolderAsSourceEntry(project.getFolder(srcFolder));
	}

	public String getOutputFolder()  {		
		try {
			String rootPath = project.getLocation().toOSString();
			String classesPath = rootPath + File.separator + ".." + File.separator + javaProject.getOutputLocation().toOSString();
			return classesPath;
		} catch (CoreException ce) {
			JonasLauncherPlugin.log("JOPE: exception getOutputFolder: " + ce );
		}
		return null;
	}
	
	public void setClassesAsOutputFolder()  throws CoreException {
//		IFolder classesFolder = this.getWebInfFolder().getFolder("classes");
//		javaProject.setOutputLocation(classesFolder.getFullPath(),null);
		
		//IFolder prjFolder = project.getFolder(project.getLocation().toOSString());
		//JonasLauncherPlugin.log("Project location = " + project.getLocation().toOSString());
		
		this.createFolder(project.getFolder("classes"));		
		IFolder classesFolder = project.getFolder("classes");
		javaProject.setOutputLocation(classesFolder.getFullPath(),null);

		}

	public void setClassesAsSourceFolder()  throws CoreException {	
		IFolder classesFolder = this.getWebInfFolder().getFolder("classes");
		this.setFolderAsSourceEntry(classesFolder);
	}


	private void createFolder(IFolder folderHandle)  throws CoreException {		
		try {
			// Create the folder resource in the workspace
			folderHandle.create(false, true, null); //new SubProgressMonitor(monitor, 500));
		}
		catch (CoreException e) {
			// If the folder already existed locally, just refresh to get contents
			if (e.getStatus().getCode() == IResourceStatus.PATH_OCCUPIED)
				folderHandle.refreshLocal(IResource.DEPTH_INFINITE, null); //new SubProgressMonitor(monitor, 500));
			else
				throw e;
		}
	}
		
	public void createSrcEJBFolder()  throws CoreException {

		//this.createFolder(this.getWebInfFolder().getFolder("src"));
		IFolder srcEJBDirFolder = this.getSrcEJBDirFolder();
		if (srcEJBDirFolder != null)
			this.createFolder(srcEJBDirFolder);
		setSrcAsSourceFolder(srcEJBDir);
	}

	private void setFolderAsSourceEntry(IFolder folderHandle) throws CoreException {
		IClasspathEntry[] entries= javaProject.getRawClasspath();
		IClasspathEntry[] newEntries= new IClasspathEntry[entries.length + 1];
		System.arraycopy(entries,0, newEntries,0, entries.length);
		newEntries[entries.length] = JavaCore.newSourceEntry(folderHandle.getFullPath());
		javaProject.setRawClasspath(newEntries, null);	
	}



	public void hotDeploy() throws CoreException {
		String baseDir = JonasLauncherPlugin.getDefault().getBaseDir();
		File directoryWeb = null;				
		if (getWebdirName().trim().equals("")) 
			directoryWeb = new File(project.getDescription().getLocation().toOSString());
		else 
			directoryWeb = new File(getWebDirFolder().getLocation().toOSString());				 					 	
		File directorySrc = null;
		if (this.getSrcDir().trim().equals("")) 
				directorySrc = new File(project.getDescription().getLocation().toOSString());
			else 
				directorySrc = new File(getSrcDirFolder().getLocation().toOSString());
					 					 				
		HotDeploy hot = new HotDeploy(baseDir, new Path(directoryWeb.toString()),getWebContext()); 
		hot.copyFileByAnt();	 					
		
	}

	/*
	 * if WEB-INF classes contains Java files add it to source folders
	 * Otherwise Eclipse will delete all those files
	 */
	private boolean classesContainsJavaFiles() {		
		IFolder webinfFolder = this.getWebInfFolder();	
		IFolder classesFolder = webinfFolder.getFolder("classes");
		File f = classesFolder.getLocation().toFile();
				 
    	if (!f.exists()) return false; 
    	if (!f.isDirectory()) return false; 
    
    	return FileUtil.dirContainsFiles(f, "java", true);
	}  

	public void setCreateSamples( boolean b )
	{
		createSamples = b;
	}

}
