package com.bull.eclipse.jonas;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */

import java.io.File;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class JonasProjectGeneralPropertyPage implements JonasPluginResources {

	private Button isJonasProjectCheck;
	private Button exportSourceCheck;
	private Text webpathText;
	private Text warLocationText;
	private Text webDirText;
	private Text srcDirText;
	private Text srcEJBDirText;



	private JonasProjectPropertyPage page;
	
	private static final int TEXT_FIELD_WIDTH = 200;
	
	public JonasProjectGeneralPropertyPage(JonasProjectPropertyPage page) {
		this.page = page;
	}
	
	
	/**
	 * returns a control which consists of the ui elements of this page
	 */
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
	
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		createIsJonasProjectGroup(composite);
//		new Label(composite, SWT.NULL); //blank

		createWebNameGroup(composite);
		createWebSrcGroup(composite);
		createWebpathGroup(composite);
//		new Label(composite, SWT.NULL); //blank
			
		createWarLocationGroup(composite);
			
		createEJBpathGroup(composite);
		
		return composite;
	}

	public void createIsJonasProjectGroup(Composite parent) {
		Composite isJonasProjectGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		isJonasProjectGroup.setLayout(layout);
		isJonasProjectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// project location entry field
		isJonasProjectCheck = new Button(isJonasProjectGroup, SWT.CHECK | SWT.LEFT);
		isJonasProjectCheck.setText(PROPERTIES_PAGE_PROJECT_ISJONASPROJECT_LABEL);
		isJonasProjectCheck.setEnabled(true);

		try {		
			isJonasProjectCheck.setSelection(page.getJavaProject().getProject().hasNature(JonasLauncherPlugin.NATURE_ID));
		} catch (CoreException ex) {
			JonasLauncherPlugin.log(ex.getMessage());	
		}
	}

	public void createWebNameGroup(Composite parent) {
			Composite webnameGroup = new Composite(parent,SWT.NONE);
			GridLayout layout = new GridLayout();
			layout.numColumns = 3;
			webnameGroup.setLayout(layout);
			webnameGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

			// location label
			Label webnameLabel = new Label(webnameGroup,SWT.NONE);
			webnameLabel.setText(WIZARD_PROJECT_WEBNAME_LABEL);
			webnameLabel.setEnabled(true);

			// project location entry field
			webDirText = new Text(webnameGroup, SWT.BORDER);
			GridData data = new GridData(GridData.FILL_HORIZONTAL);
			data.widthHint = TEXT_FIELD_WIDTH;
//			data.horizontalSpan = 2;
			webDirText.setLayoutData(data);		
			webDirText.setText(this.getwebDir()); 
			webDirText.setEnabled(true);
	}

	public void createWebSrcGroup(Composite parent) {
			Composite websrcGroup = new Composite(parent,SWT.NONE);
			GridLayout layout = new GridLayout();
			layout.numColumns = 3;
			websrcGroup.setLayout(layout);
			websrcGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

												
				//		location label
			 Label websrcLabel = new Label(websrcGroup,SWT.NONE);
			 websrcLabel.setText(WIZARD_PROJECT_WEBSRC_LABEL);
			 websrcLabel.setEnabled(true);

				 // project location entry field
			 srcDirText = new Text(websrcGroup, SWT.BORDER);
			 GridData data1 = new GridData(GridData.FILL_HORIZONTAL);
			 data1.widthHint = TEXT_FIELD_WIDTH;
//					 data.horizontalSpan = 2;
			 srcDirText.setLayoutData(data1);		
			 srcDirText.setText(this.getsrcDir()); 
			 srcDirText.setEnabled(true);
		}


	public void createWebpathGroup(Composite parent) {
		Composite webpathGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		webpathGroup.setLayout(layout);
		webpathGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// location label
		Label webpathLabel = new Label(webpathGroup,SWT.NONE);
		webpathLabel.setText(WIZARD_PROJECT_WEBPATH_LABEL);
		webpathLabel.setEnabled(true);

		// project location entry field
		webpathText = new Text(webpathGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
//		data.horizontalSpan = 2;
		webpathText.setLayoutData(data);		
		webpathText.setText(this.getWebPath()); 
		webpathText.setEnabled(true);
								
	}
	
	
	
	public void createWarLocationGroup(Composite parent) {
		Composite warLocationGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		warLocationGroup.setLayout(layout);
		warLocationGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// location label
		Label warLocationLabel = new Label(warLocationGroup,SWT.NONE);
		warLocationLabel.setText(WIZARD_PROJECT_WARLOCATION_LABEL);
		warLocationLabel.setEnabled(true);

		// project location entry field
		warLocationText = new Text(warLocationGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
		warLocationText.setLayoutData(data);		
		warLocationText.setText(this.getWarLocation()); 
		warLocationText.setEnabled(true);
		
		Button browseButton = new Button(warLocationGroup, SWT.PUSH);
		browseButton.setText(BROWSE_BUTTON_LABEL);
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent evt) {
				String newValue = warFieldChange();
				if (newValue != null) {
					warLocationText.setText(newValue);
				}
			}
		});

		browseButton.setEnabled(true);
		
		exportSourceCheck = new Button(warLocationGroup, SWT.CHECK | SWT.LEFT);
		exportSourceCheck.setText(WIZARD_PROJECT_EXPORTSOURCE_LABEL);
		data = new GridData();
		data.horizontalSpan = 3;
		exportSourceCheck.setLayoutData(data);		
		exportSourceCheck.setEnabled(true);
		exportSourceCheck.setSelection(this.getExportSource());
	}

	public void createEJBpathGroup(Composite parent) {
		Composite ejbpathGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		ejbpathGroup.setLayout(layout);
		ejbpathGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// location label
		Label ejbpathLabel = new Label(ejbpathGroup,SWT.NONE);
		ejbpathLabel.setText(WIZARD_PROJECT_EJBSRC_LABEL);
		ejbpathLabel.setEnabled(true);

		// project location entry field
		srcEJBDirText = new Text(ejbpathGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
//		data.horizontalSpan = 2;
		srcEJBDirText.setLayoutData(data);		
		srcEJBDirText.setText(this.getSrcEJBDir()); 
		srcEJBDirText.setEnabled(true);
								
	}

	protected String getWebPath() {
		String result = "";
		try {
			JonasProject prj = page.getJonasProject();
			if(prj != null) 
				result = prj.getWebContext();						
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}
	
	protected String getWarLocation() {
		String result = "";
		try {
			JonasProject prj = page.getJonasProject();
			if(prj != null) {			
				result = prj.getWarLocation();
				if (result.trim().equals("")) 			
					result = prj.getWebContext() + ".war";
			}
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}

	protected String getwebDir() {
		String result = "";
		try {
			JonasProject prj = page.getJonasProject();
			if(prj != null) {			
				result = prj.getWebdirName();
				//if (result.trim().equals("")) 			
				//	result = "webApplication";
			}
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}

	protected String getsrcDir() {
		String result = "";
		try {
			JonasProject prj = page.getJonasProject();
			if(prj != null) {			
				result = prj.getSrcDir();
				//if (result.trim().equals("")) 			
				//	result = "source";
			}
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}

	protected String getSrcEJBDir() {
		String result = "";
		try {
			JonasProject prj = page.getJonasProject();
			if(prj != null) {			
				result = prj.getSrcEJBDir();
				//if (result.trim().equals("")) 			
				//	result = "source";
			}
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}

	protected boolean getExportSource() {
		boolean result = false;
		try {
			JonasProject prj = page.getJonasProject();
			if(prj != null)
				result = prj.getExportSource();
		} catch (CoreException ex) {
			// result = false;
		}
		return result;
	}

				
	/**
	 * performes the ok action for this property page
	 */
	public boolean performOk() {
		try {
			if(isJonasProjectCheck.getSelection()) {		
				JonasProject.addJonasNature(page.getJavaProject());
				JonasProject prj = page.getJonasProject();
				//prj.updateWebPath(webpathText.getText());
				prj.setWebContext(webpathText.getText());
				prj.setWebdirName(webDirText.getText());
				prj.setSrcDir(srcDirText.getText());
				prj.setWarLocation(warLocationText.getText());
				//prj.updateWebPath(webpathText.getText());				
				prj.setExportSource(exportSourceCheck.getSelection());
				prj.setSrcEJBDir(srcEJBDirText.getText());				
				prj.saveProperties();
			} else {
				//page.getJonasProject().removeContextInServerXML();
				JonasProject.removeJonasNature(page.getJavaProject());
			}
		} catch (Exception ex) {
			JonasLauncherPlugin.log(ex.getMessage());	
		}
		
		return true;
	}
	
	
	
	protected String warFieldChange() {
		File f = new File(warLocationText.getText());
		if (!f.exists())
			f = null;
		File d = getFile(f);
		if (d == null)
			return null;
	
		return d.getAbsolutePath();
	}
	
	public boolean isJonasProjectChecked() {
		return isJonasProjectCheck.getSelection();
	}
	
	/**
	 * Helper to open the file chooser dialog.
	 */
	private File getFile(File startingDirectory) {
	
		FileDialog dialog = new FileDialog(page.getShell(), SWT.OPEN);
		if (startingDirectory != null)
			dialog.setFileName(startingDirectory.getPath());
	//	if (extensions != null)
	//		dialog.setFilterExtensions(extensions);
		String file = dialog.open();
		if (file != null) {
			file = file.trim();
			if (file.length() > 0)
				return new File(file);
		}
	
		return null;
	}

}
