package com.bull.eclipse.jonas;

/*
 * (c) Copyright Emmanuel Rias, Bull SA 2003.
 * All Rights Reserved.
 */
 
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.util.TabFolderLayout;
import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.dialogs.PropertyPage;

/**
 * provides a tab control with property pages for the 
 * settings of a jonas project.
 * 
 * @version 	1.0
 * @author		Emmanuel Rias
 */

public class JonasProjectPropertyPage extends PropertyPage 
	implements IWorkbenchPreferencePage, JonasPluginResources 
{
	private static final boolean DO_NOT_USE_WEBAPP_CLASSLOADER = false;
	
	private TabFolder 							  folder;
	private JonasProjectGeneralPropertyPage	  generalPropertyPage;
	private JonasProjectExtraResourcesPropertyPage extraResourcesPropertyPage;

	/**
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		if (DO_NOT_USE_WEBAPP_CLASSLOADER==false) {
			folder = new TabFolder(parent, SWT.NONE);
		
			folder.setLayout(new TabFolderLayout());	
			folder.setLayoutData(new GridData(GridData.FILL_BOTH));	
		
			// general property page
			generalPropertyPage = new JonasProjectGeneralPropertyPage(this);		
			// add to tab
			TabItem generalTab = new TabItem(folder, SWT.NONE);
			generalTab.setText(JonasPluginResources.PROPERTIES_PAGE_PROJECT_GENERAL_TAB_LABEL);		
			generalTab.setControl(generalPropertyPage.createContents(folder));
			
			// extra resources property page
			extraResourcesPropertyPage = new JonasProjectExtraResourcesPropertyPage(this);
			// add to tab
			TabItem extraClassTab = new TabItem(folder, SWT.NONE);
			extraClassTab.setText(JonasPluginResources.PROPERTIES_PAGE_PROJECT_EXTRA_TAB_LABEL);		
			extraClassTab.setControl(extraResourcesPropertyPage.getControl(folder));
			
			return folder;		
		} else {
			generalPropertyPage = new JonasProjectGeneralPropertyPage(this);
			return generalPropertyPage.createContents(parent);
		}
	}
	
	/**
	 * @see IPreferencePage#performOk()
	 */
	public boolean performOk() {
		// delegate to property pages
		if (generalPropertyPage.performOk() == true) 
			// check if it's a jonas resources any more 
			if (generalPropertyPage.isJonasProjectChecked()) 
				if (extraResourcesPropertyPage != null) 
					extraResourcesPropertyPage.performOk();								

		return true;
	}	
	
	/**
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	/* helper methods */
	protected IJavaProject getJavaProject() throws CoreException {
		IProject project = (IProject)(this.getElement().getAdapter(IProject.class));
		return (IJavaProject)(project.getNature(JavaCore.NATURE_ID));
	}
	protected JonasProject getJonasProject() throws CoreException {
		return JonasProject.create(getJavaProject());
	}	
	
}

