package com.bull.eclipse.jonas.actions;

import java.util.Iterator;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bull.eclipse.jonas.JonasLauncherPlugin;

abstract public class JonasFileAbstractActionDelegate
	implements IWorkbenchWindowActionDelegate {
	protected IWorkbenchWindow window;
	private String msg;
	private Iterator it = null;
	private ICompilationUnit compunit = null;

	/*
	 * @see IWorkbenchWindowActionDelegate#dispose()
	 */

	public void dispose() {
	}

	/*
	 * @see IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	/*
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = JavaPlugin.getActiveWorkbenchShell();
		try {

			String actionId = action.getId();
			if (actionId.equalsIgnoreCase("EJB.addLog")) {
				it = getIterator();
				if (it == null)
					return;
				while (it.hasNext()) {
					Object compunit = it.next();
					if (compunit instanceof ICompilationUnit
						&& compunit != null) {
						
						if (!this.doActionOn((ICompilationUnit) compunit)) {
							setMsgToFail("Erreur ..", true);
							MessageDialog.openInformation(shell, "Jonas", msg);
							return;
						}
					}
				}
				
				setMsgToSuccess();
				MessageDialog.openInformation(shell, "Jonas", msg);
				
			}
			
			else  {
			compunit = this.getCurrentSelection();
			if ((compunit != null)) {
				if (this.doActionOn(compunit)) {
					setMsgToSuccess();
					MessageDialog.openInformation(shell, "Jonas", msg);
				}
			}
		}  
			
		} catch (Exception ex) {
			System.out.println("Exception : " + ex.getMessage());
			JonasLauncherPlugin.log(ex.getStackTrace().toString());
			setMsgToFail(ex.getMessage(), true);
			MessageDialog.openInformation(shell, "Jonas", msg);
		}

	}

	protected ICompilationUnit getCurrentSelection() {

		IWorkbenchWindow window = JavaPlugin.getActiveWorkbenchWindow();
		ICompilationUnit result = null;

		if (window != null) {
			ISelection selection = window.getSelectionService().getSelection();
			if (selection instanceof IStructuredSelection) {
				Object compunit =
					((IStructuredSelection) selection).getFirstElement();
				if (compunit instanceof ICompilationUnit)
					result = (ICompilationUnit) compunit;
			}
		}
		return result;
	}

	protected Iterator getIterator() {

		IWorkbenchWindow window = JavaPlugin.getActiveWorkbenchWindow();
		ICompilationUnit result = null;

		if (window != null) {
			ISelection selection = window.getSelectionService().getSelection();
			if (selection instanceof IStructuredSelection) {
				return ((IStructuredSelection) selection).iterator();
			}
		}
		return null;
	}

	protected ICompilationUnit getNextSelection() {

		IWorkbenchWindow window = JavaPlugin.getActiveWorkbenchWindow();
		ICompilationUnit result = null;

		if (window != null) {
			ISelection selection = window.getSelectionService().getSelection();
			if (selection instanceof IStructuredSelection) {
				Object compunit =
					((IStructuredSelection) selection).getFirstElement();
				if (compunit instanceof ICompilationUnit)
					result = (ICompilationUnit) compunit;
			}
		}
		return result;
	}

	abstract public boolean doActionOn(ICompilationUnit compunit)
		throws Exception;

	/**
	 * Sets the msg.
	 * @param msg The msg to set
	 */
	private void setMsgToFail(String detail, boolean seelog) {
		this.msg = JonasLauncherPlugin.getResourceString("msg.action.failed");
		this.msg += "\n" + detail;
		if (seelog) {
			this.msg
				+= JonasLauncherPlugin.getResourceString("msg.action.seelog");
		}
	}

	/**
	 * Sets the msg.
	 * @param msg The msg to set
	 */
	private void setMsgToSuccess() {
		this.msg =
			JonasLauncherPlugin.getResourceString("msg.action.succeeded");
	}

	/*
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {

	}

}
