package com.bull.eclipse.jonas.actions;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */
 

import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasProject;

abstract public class JonasProjectAbstractActionDelegate implements IWorkbenchWindowActionDelegate {
	protected IWorkbenchWindow window;
	private String msg;

	/*
	 * @see IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
	}

	/*
	 * @see IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	/*
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {			
		Shell shell= JavaPlugin.getActiveWorkbenchShell();
		try {
			JonasProject prj = this.getCurrentSelection();			
			String actionId = action.getId();
			if((prj != null) ) {
				if (this.doActionOn(prj)) {			
					setMsgToSuccess();
					MessageDialog.openInformation(shell,"Jonas", msg);			
				}
			}
		} catch (Exception ex) {			
			JonasLauncherPlugin.log(ex.getStackTrace().toString());			
			setMsgToFail(ex.getMessage(), true);
			MessageDialog.openInformation(shell,"Jonas", msg);
		} 
		
	}

	/*
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {

	}

	protected JonasProject getCurrentSelection() {
		IWorkbenchWindow window = JavaPlugin.getActiveWorkbenchWindow();
		JonasProject result = null;
		if (window != null) {			
			ISelection selection= window.getSelectionService().getSelection();
			if (selection instanceof IStructuredSelection) {
				Object project = ((IStructuredSelection)selection).getFirstElement();
				if(project instanceof IProject)
					result = JonasProject.create((IProject)project);
				if(project instanceof IJavaProject)
					result = JonasProject.create((IJavaProject)project);
			}						
		}
		return result;
	}
	
	abstract public boolean doActionOn(JonasProject prj) throws Exception;

	/**
	 * Sets the msg.
	 * @param msg The msg to set
	 */
	private void setMsgToFail(String detail, boolean seelog) {
		this.msg = JonasLauncherPlugin.getResourceString("msg.action.failed");
		this.msg += "\n" + detail;
		if(seelog) {	
			this.msg += JonasLauncherPlugin.getResourceString("msg.action.seelog"); 
		}
	}
	
	/**
	 * Sets the msg.
	 * @param msg The msg to set
	 */
	private void setMsgToSuccess() {
		this.msg = JonasLauncherPlugin.getResourceString("msg.action.succeeded");
	}	
}

