package com.bull.eclipse.jonas.actions.ws;

import java.io.File;

import org.eclipse.core.resources.IResource;
import org.eclipse.jdt.core.IPackageFragment;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.actions.JonasPackageAbstractActionDelegate;
import com.bull.eclipse.jonas.utils.EjbManagement;
import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;
import com.bull.eclipse.jonas.utils.xml.FileXmlUtils;
import com.bull.eclipse.jonas.utils.xml.UpdateXmlFile;



public class CreateWSActionDelegate extends JonasPackageAbstractActionDelegate
{

	public boolean doActionOn(IPackageFragment pack) throws Exception {
		// Update build.properties with JONAS_ROOT
		
		JonasProject jp = (JonasProject)pack.getJavaProject().getProject().getNature(JonasLauncherPlugin.NATURE_ID);
		String projDir =
			pack
				.getJavaProject()
				.getProject()
				.getParent()
				.getLocation()
				.toOSString();
		String packName = pack.getPath().removeFirstSegments(2).toOSString();
		updateProjectWithAxisConfig(jp);
		File serverConfigFile =
			new File(
				jp
					.getWebInfFolder()
					.getLocation()
					.append("server-config.wsdd")
					.toOSString());
		createDeployWSDD(jp, packName);
		if (!serverConfigFile.exists())
			FileUtil.copy(new File(TemplateDirUtil.getTemplateDir() + File.separator + "ws" + File.separator + "server-config.wsdd"), new File(jp.getWebInfFolder().getLocation().append("server-config.wsdd").toOSString()));
		updateServerConfig(jp, packName, EjbManagement.getEjbName(packName,jp));
		jp.getProject().refreshLocal( IResource.DEPTH_INFINITE, null );			 

		return false;
	}			

	private void updateProjectWithAxisConfig(JonasProject jp) {
		UpdateXmlFile.updateWebXmlFileWithAxisServlet(jp.getWebInfFolder().getLocation().append("web.xml").toFile());		
	}

	private boolean createDeployWSDD(JonasProject jp, String packName) {
		CreateDeployWSDD usc = new CreateDeployWSDD(jp);
		String ejbName = EjbManagement.getEjbName(packName,jp);
		JonasLauncherPlugin.log("Xml File = " + jp.getSrcEJBDirFolder().getLocation().append(packName).append("jonas-" + ejbName + ".xml"));
		File jonasXmlFile = new File(jp.getSrcEJBDirFolder().getLocation().append(packName).append("jonas-" + ejbName + ".xml").toOSString());
		File standardXmlFile = new File(jp.getSrcEJBDirFolder().getLocation().append(packName).append(ejbName + ".xml").toOSString());
		if (!jonasXmlFile.exists()) 
			return false; 
		String jndiName = FileXmlUtils.getValueForNode(jonasXmlFile,"jndi-name");
		String homeClass = FileXmlUtils.getValueForNode(standardXmlFile,"home");
		String remoteClass = FileXmlUtils.getValueForNode(standardXmlFile,"remote");
		String homeLocalClass = FileXmlUtils.getValueForNode(standardXmlFile,"local-home");
		String localClass = FileXmlUtils.getValueForNode(standardXmlFile,"local");
		
		usc.runUpdate(ejbName + "EndPoint",jndiName,homeClass,remoteClass,homeLocalClass,localClass,packName);
		return false;
	}

	private boolean updateServerConfig(JonasProject jp, String packName, String jarName) {
		UpdateXmlFile.updateServerConfigXmlFile(packName,jp, jarName);		
		return false;
	}

}

