/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s):  Philippe Coq
 * --------------------------------------------------------------------------
 * $Id: JndiEnvRefsGroupDesc.java,v 1.5 2003/12/05 12:24:15 sauthieg Exp $
 * --------------------------------------------------------------------------
 */


package com.bull.eclipse.jonas.utils.xml;

// import java
import java.util.Vector;


/**
 * This class is used to keep the usage of containedJNDI environment
 * regerence elements consistent across J2EE deployment descriptors
 * Elements are "env-entry", "ejb-ref", "ejb-local-ref", "resource-ref",
 * "resource-env-ref, "service-ref"" 
 * This class extends DescriptionGroupDesc  and give access to elements
 * "description", "display-name", "icon"
 * used by entity, session, message-driven-bean, web-app, clientApp
 * 
 * @author Philippe Coq
 * 
 */

public abstract class JndiEnvRefsGroupDesc extends DescriptionGroupDesc 
    implements JndiEnvRefsGroup  {

    /** 
     * List of resource env ref Descriptors.
     */
    private Vector resourceEnvRefs = new Vector();

    /**
     * List of resource ref Descriptors..
     */
    private Vector resourceRefs = new Vector();

    /**
     * List of env-entry Descriptors.
     */
    private Vector envEntries = new Vector();

    /**
     * List of service-ref Descriptors.
     */
    private Vector serviceRefs = new Vector();

    /**
     * List of EJB ref Descriptors..
     */
    private Vector ejbRefs = new Vector();

    /**
     * List of local EJB ref Descriptors..
     */
    private Vector ejbLocalRefs = new Vector();

   /**
     * Construct an instance of a JndiEnvRefsGroupDesc.<BR>
     * 
     * @param classLoader the classloader for the classes.
     * @param app  the data structure of the J2EE application
     * @param jonasApp the data structure of the jonas specific deployment descriptor
     * @param fileName the name of the warFile (useful only for service-ref)
     * @throws DeploymentDescException if the deployment 
     * descriptors are corrupted.
     */
//    protected JndiEnvRefsGroupDesc(ClassLoader classLoader,
//                                   JndiEnvRefsGroupXml app,
//                                   JonasJndiEnvRefsGroupXml jonasApp,
//                                   String fileName)
//      throws DeploymentDescException {  
//
//        // test classloader
//        if (classLoader == null) {
//            throw new DeploymentDescException("Classloader is null");
//        }
//        // display name
//        displayName = (app.getDisplayName() != null) ? app.getDisplayName() : null;
//
//        // resource-env-ref
//        List resourceEnvRefList = app.getResourceEnvRefList();
//        List jonasResourceEnvList = jonasApp.getJonasResourceEnvList();
//        HashMap resourceEnv = new HashMap();
//        for (Iterator i = jonasResourceEnvList.iterator(); i.hasNext();) {
//            JonasResourceEnv jonasResourceEnv = (JonasResourceEnv) i.next();
//            String rName = jonasResourceEnv.getResourceEnvRefName();
//            boolean findResRef = false;
//            for (Iterator j = resourceEnvRefList.iterator(); j.hasNext() && !findResRef;) {
//                ResourceEnvRef resourceEnvRef = (ResourceEnvRef) j.next();
//                if (rName.equals(resourceEnvRef.getResourceEnvRefName())) {
//                    findResRef = true;
//                }
//            }
//            if (!findResRef) {
//                throw new DeploymentDescException("resource-env-ref missing for jonas-resource-env " + rName);
//            }
//            resourceEnv.put(rName, jonasResourceEnv);
//        }
//        for (Iterator i = resourceEnvRefList.iterator(); i.hasNext();) {
//            ResourceEnvRef resourceEnvRef = (ResourceEnvRef) i.next();
//            String rName = resourceEnvRef.getResourceEnvRefName();
//            if (!resourceEnv.containsKey(rName)) {
//                throw new DeploymentDescException("jonas-resource-env missing for resource-env-ref-name " + rName);
//            }
//            resourceEnvRefs.addElement(new ResourceEnvRefDesc(classLoader, 
//                                                              resourceEnvRef, 
//                                                              (JonasResourceEnv) resourceEnv.get(rName)));
//        }
//
//        // resource-ref
//        List resourceRefList = app.getResourceRefList();
//        List jonasResourceList = jonasApp.getJonasResourceList();
//        HashMap resource = new HashMap();
//        for (Iterator i = jonasResourceList.iterator(); i.hasNext();) {
//            JonasResource jonasResource = ((JonasResource) i.next());
//            String rName = jonasResource.getResRefName();
//            boolean findResRef = false;
//            for (Iterator j = resourceRefList.iterator(); j.hasNext() && !findResRef;) {
//                if (rName.equals(((ResourceRef) j.next()).getResRefName())) {
//                    findResRef = true;
//                }
//            }
//            if (!findResRef) {
//                throw new DeploymentDescException("resource-ref missing for jonas-resource " + rName);
//            }
//            resource.put(rName, jonasResource);
//        }
//
//        for (Iterator i = resourceRefList.iterator(); i.hasNext();) {
//            ResourceRef resourceRef = ((ResourceRef) i.next());
//            String rName = resourceRef.getResRefName();
//            if (!resource.containsKey(rName)) {
//                throw new DeploymentDescException("jonas-resource missing for res-ref-name " + rName);
//            }
//            resourceRefs.addElement(new ResourceRefDesc(classLoader, resourceRef, (JonasResource) resource.get(rName)));
//        }
//
//        // env-entry
//        for (Iterator i = app.getEnvEntryList().iterator(); i.hasNext();) {
//            EnvEntryDesc envEntryDesc = new EnvEntryDesc((EnvEntry) i.next());
//            envEntries.addElement(envEntryDesc);
//        }
//
//        // ejb-ref
//        List ejbRefList = app.getEjbRefList();
//        List jonasEjbRefList = jonasApp.getJonasEjbRefList();
//
//        HashMap ejbRef = new HashMap();
//        for (Iterator i = jonasEjbRefList.iterator(); i.hasNext();) {
//            JonasEjbRef jonasEjbRef = (JonasEjbRef) i.next();
//            String brName = jonasEjbRef.getEjbRefName();
//            boolean findBRef = false;
//            for (Iterator j = ejbRefList.iterator(); j.hasNext() && !findBRef;) {
//                EjbRef eRef = (EjbRef) j.next();
//                if (brName.equals(eRef.getEjbRefName())) {
//                    findBRef = true;
//                }
//            }
//            if (!findBRef) {
//                throw new DeploymentDescException("ejb-ref missing for jonas-ejb-ref " + brName);
//            }
//            ejbRef.put(brName, jonasEjbRef);
//        }
//        for (Iterator i = ejbRefList.iterator(); i.hasNext();) {
//            EjbRef eRef = (EjbRef) i.next();
//            String brName = eRef.getEjbRefName();
//            JonasEjbRef jonasEjbRef = (JonasEjbRef) ejbRef.get(brName);
//            if (jonasEjbRef == null) {
//                // The ejb-link must be specified when there is no jonas-ejb-ref
//                if ((eRef.getEjbLink() == null)) {
//                    // Ignore the ejb-link syntax "product.jar#ProductEJB"
//                    throw new DeploymentDescException("ejb-link missing for ejb-ref-name " + brName);
//                }
//            }
//            ejbRefs.addElement(new EjbRefDesc(eRef, jonasEjbRef));
//        }
//
//
//        // ejb-local-ref
//        List ejbLocalRefList = app.getEjbLocalRefList();
//        for (Iterator i = ejbLocalRefList.iterator(); i.hasNext();) {
//            EjbLocalRef ejbLocalRef = (EjbLocalRef) i.next();
//            if ((ejbLocalRef.getEjbLink() == null)) {
//                throw new DeploymentDescException("ejb-link missing for ejb-ref-name " + ejbLocalRef.getEjbRefName());
//            }
//            ejbLocalRefs.addElement(new EjbLocalRefDesc(ejbLocalRef));
//        }
//
//       // service-ref
//        if ( fileName != null) {
//            List serviceRefList = app.getServiceRefList();
//            List jonasServiceRefList = jonasApp.getJonasServiceRefList();
//            Hashtable service = new Hashtable();
//            for (Iterator i = jonasServiceRefList.iterator(); i.hasNext();) {
//                JonasServiceRef jonasServiceRef = ((JonasServiceRef) i.next());
//                String svcName = jonasServiceRef.getServiceRefName();
//                boolean findServiceRef = false;
//                for (Iterator j = serviceRefList.iterator(); j.hasNext() && !findServiceRef;) {
//                    if (svcName.equals(((ServiceRef) j.next()).getServiceRefName())) {
//                        findServiceRef = true;
//                    }
//                }
//                if (!findServiceRef) {
//                    throw new DeploymentDescException("service-ref missing for jonas-service-ref " + svcName);
//                }
//                service.put(svcName, jonasServiceRef);
//            }
//            
//            for (Iterator i = serviceRefList.iterator(); i.hasNext();) {
//                ServiceRef serviceRef = ((ServiceRef) i.next());
//                String svcName = serviceRef.getServiceRefName();
//                if (!service.containsKey(svcName)) {
//                    throw new DeploymentDescException("jonas-service-ref missing for service-ref-name " + svcName);
//                }
//                serviceRefs.addElement(new ServiceRefDesc(classLoader,
//                                                          serviceRef,
//                                                          (JonasServiceRef) service.get(svcName),
//                                                          fileName));
//            }
//            
//            
//            // check unicity of handler-name (must be unique in the module)
//            List hNames = new Vector();
//            for (Iterator i = serviceRefs.iterator(); i.hasNext(); ) {
//                ServiceRefDesc sr = (ServiceRefDesc) i.next();
//                for (Iterator h = sr.getHandlerRefs().iterator(); h.hasNext(); ) {
//                    HandlerDesc hr = (HandlerDesc) h.next();
//                    // if name is already in the list, unicity is not respected
//                    if (hNames.contains(hr.getName())) {
//                        throw new DeploymentDescException("handler-name '" + hr.getName()
//                                                          + "' must be unique in the module '");
//                    }
//                    hNames.add(hr.getName());
//                }
//            }
//        }
//
//    }
                                   

    /**
     * Get  resource environment references.
     * @return array of resource environment reference descriptors 
     */
    public ResourceEnvRefDesc[] getResourceEnvRefDesc() {
        ResourceEnvRefDesc[] ret = new ResourceEnvRefDesc[resourceEnvRefs.size()];
        resourceEnvRefs.copyInto(ret);
        return ret;
    }
    
    /**
     * Get resource manager connection factory references.
     * @return array of resource reference descriptors 
     */
    public ResourceRefDesc[] getResourceRefDesc() {
        ResourceRefDesc[] ret = new ResourceRefDesc[resourceRefs.size()];
        resourceRefs.copyInto(ret);
        return ret;
    }

    /**
     * Get environment entries.
     * @return array of Env entries descriptors 
     */
    public EnvEntryDesc[] getEnvEntryDesc() {
        EnvEntryDesc[] ret = new EnvEntryDesc[envEntries.size()];
        envEntries.copyInto(ret);
        return ret;
    }

    /**
     * Get EJB references.
     * @return array of EJB reference descriptors
     */
    public EjbRefDesc[] getEjbRefDesc() {
        EjbRefDesc[] ret = new EjbRefDesc[ejbRefs.size()];
        ejbRefs.copyInto(ret);
        return ret;
    }

    /**
     * Get ejb local references.
     * @return array of ejb local reference descriptors 
     */
    public EjbLocalRefDesc[] getEjbLocalRefDesc() {
        EjbLocalRefDesc[] ret = new EjbLocalRefDesc[ejbLocalRefs.size()];
        ejbLocalRefs.copyInto(ret);
        return ret;
    }



    /**
     * Get service references.
     * @return array of service references descriptors 
     */
//    public ServiceRefDesc[] getServiceRefDesc() {
//        ServiceRefDesc[] ret = new ServiceRefDesc[serviceRefs.size()];
//        serviceRefs.copyInto(ret);
//        return ret;
//    }
}
