package com.bull.eclipse.newbean;


import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;



/**
 * This class is a &ldquo;bean generator&rdquo;. It uses Velocity and
 * a set of template files to generate some files that can serve as a
 * startup point for the developpement of a new bean.
 */
public class JopeUpdateBean {

    private static final int EXIT_SUCCESS = 0;
    private static final int EXIT_FAILURE = 1;

    private VelocityEngine  vEngine  = null;
    private VelocityContext vContext = null;
	private Vector pkgListe = null;
	private JonasProject prj = null;


    public JopeUpdateBean(Vector pkgListe, JonasProject prj) {
    	this.pkgListe = pkgListe;
    	this.prj = prj;
    }


    /**
     * Generates files for this new bean.
     */
    public void generate() {
        try {
			vEngine = new VelocityEngine();
			vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
			vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
			vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
					TemplateDirUtil.getTemplateDir().concat("/newbean"));
			try {
				vEngine.init();
			} catch (Exception e) {
				fatalError("unable to initilise Velocity engine (" + e + ")");
			}

			// Creates a Velocity context and populates it by walking
			// through the parameter set
			vContext = new VelocityContext();
			vContext.put("pkgListe", pkgListe);			
			generate("buildgene.vm", prj.getProject().getLocation().append("components").append("build.xml").toOSString());
            System.out.println("Your bean files have been created. You can now customize them.");
        } catch (Exception e) {
            error(e.toString());
        }
	
    }



    /**
     * Generates a file from the specified template.
     * @param templateFileName the name of the template file
     * @param targetFileName the name of the generated file
     */
    private void generate(String templateFileName,
                          String targetFileName) throws Exception, IOException, ResourceNotFoundException, ParseErrorException, MethodInvocationException {
        FileWriter fileWriter = null;
        fileWriter = new FileWriter(targetFileName);
        vEngine.mergeTemplate(templateFileName, vContext, fileWriter);
        fileWriter.close();
    }


    /**
     * Display the specified error message.
     * @param errMsg the error message to display
     */
    static void error(String errMsg) {
        System.err.println("NewBean error: " + errMsg);
    }


    /**
     * Display the specified error message and exits with an
     * EXIT_FAILURE status.
     * @param errMsg the error message to display
     */
    static void fatalError(String errMsg) {
        System.err.println("NewBean fatal error: " + errMsg);
        System.exit(EXIT_FAILURE);
    }

}
