/*
 * Created on Sep 22, 2003
 *
 */
 
package com.bull.eclipse.newbean;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;
import com.bull.eclipse.jonas.wizard.CreateEJBWizardPage;


/**
 * Interface with the JOnAS 'newbean' functionality.
 * @author steve krupanszky
 *
 */
public class NewbeanRunner
{
	private JonasProject	jonasNature;
	private IFolder			pkgfolder;	// the folder in which the java files will go
	private JonasProject		project;

	private VelocityEngine vEngine = null;
	private VelocityContext vContext = null;
	
	private static final int EXIT_SUCCESS = 0;
	private static final int EXIT_FAILURE = 1;

    private final Integer SESSION_BEAN        = new Integer(0);
    private final Integer ENTITY_BEAN         = new Integer(1);
    private final Integer MESSAGE_DRIVEN_BEAN = new Integer(2);
    private final Integer BMP      = new Integer(0);
    private final Integer CMP1     = new Integer(1);
    private final Integer CMP2     = new Integer(2);
    private final Integer STATELESS = new Integer(0);
    private final Integer STATEFUL  = new Integer(1);
    private final Boolean FALSE  = Boolean.FALSE;
    private final Boolean TRUE   = Boolean.TRUE;



	
	
	public			NewbeanRunner( JonasProject proj )
	{
		project = proj;
	}

	/**
	 * Copy file into package, if it doesn't already exist there.
	 * Package must already exist.
	 */
	private void	copyFileToPackage( File f, String p )
	{
		IFile	javafile = pkgfolder.getFile( f.getName() );
		if( javafile.getLocation().toFile().exists() ) {
			JonasLauncherPlugin.log(
				"JOPE: file " + p + "/" + f.getName()
				+ " already exists in project " + project.getProject().getName() );
		}
		else {
			try{ FileUtil.copy( f, javafile.getLocation().toFile() ); }
			catch( IOException e ) {
				JonasLauncherPlugin.log(
					"JOPE: exception copying file " + p + "/" + f.getName()
					+ " to project " + project.getProject().getName() + ": " + e );
			}
		}
	}

	private void	copyFileToComponents(File f)
	{
		File javafile = project.getProject().getLocation().append(project.getSrcEJBDir()).append("build.xml").toFile();
		try{ FileUtil.copy( f, javafile ); }
		catch( IOException e ) {
				JonasLauncherPlugin.log(
					"JOPE: exception copying file " );
		}
	}

	/**
	 * Copy file into project, if it doesn't already exist there.
	 */
	private void	copyFileToProject( File f )
	{
		IFile	xmlfile = project.getProject().getFile( f.getName() );
		if( xmlfile.getLocation().toFile().exists() ) {
			JonasLauncherPlugin.log("JOPE: file " + f.getName()
				+ " already exists in project " + project.getProject().getName() );
		}
		else {
			try{ FileUtil.copy( f, xmlfile.getLocation().toFile() ); }
			catch( IOException e ) {
				JonasLauncherPlugin.log(
					"JOPE: exception copying file " + f.getName()
					+ " to project " + project.getProject().getName() + ": " + e );
			}
		}
	}

	private boolean		myDelete( File f )
	{
		boolean		b = f.delete();
		int			retryCount = 5;
		while( !b && retryCount-->0 ) {
			try{ java.lang.Thread.sleep(1000); } catch(Exception e) {}
			b = f.delete();
		}
		return b;
	}

	public void		RunNewbean( CreateEJBWizardPage EjbPage )
	{
		String jonasRoot = JonasLauncherPlugin.getDefault().getJonasDir();
		vContext = new VelocityContext();
		
		String		beanName = EjbPage.getEJBName();
		vContext.put("beanName",beanName);
		String		pkgName = EjbPage.getPkgName();
		vContext.put("pkgName",pkgName);
		String		jarName = EjbPage.getJarName();
		Integer beanFlavor = null;
		if( jarName.length()>0 )
			vContext.put("jarName",jarName);
		String s = EjbPage.getEJBType();
		if( s.equals(CreateEJBWizardPage.STATEFUL) ) {
			vContext.put("beanType","F");
			vContext.put("sessionType",STATEFUL);	
			beanFlavor = SESSION_BEAN;
	        vContext.put("beanFlavor", beanFlavor);
		} else
		if( s.equals(CreateEJBWizardPage.STATELESS) ) {
			vContext.put("beanType","L");
			vContext.put("sessionType",STATELESS);
			beanFlavor = SESSION_BEAN;
	        vContext.put("beanFlavor", beanFlavor);
		} else
		if( s.equals(CreateEJBWizardPage.ENTITYb) ) {
			vContext.put("beanType","E");
			vContext.put("sessionType","B");	
            beanFlavor = ENTITY_BEAN;
	        vContext.put("beanFlavor", beanFlavor);
            vContext.put("persistenceManager", BMP);
		} else
		if( s.equals(CreateEJBWizardPage.ENTITYc1) ) {
			vContext.put("beanType","E");
			vContext.put("sessionType","C");	
            beanFlavor = ENTITY_BEAN;
	        vContext.put("beanFlavor", beanFlavor);
            vContext.put("persistenceManager", CMP1);
		} else
		if( s.equals(CreateEJBWizardPage.ENTITYc2) ) {
			vContext.put("beanType","E");
			vContext.put("sessionType","C2");	
            beanFlavor = ENTITY_BEAN;
	        vContext.put("beanFlavor", beanFlavor);
            vContext.put("persistenceManager", CMP2);
		} else
		if( s.equals(CreateEJBWizardPage.MDB) ) {
			vContext.put("beanType","MD");
            beanFlavor = MESSAGE_DRIVEN_BEAN;
	        vContext.put("beanFlavor", beanFlavor);
		}
		s = EjbPage.getPrimaryKey();
		if( s.equals(CreateEJBWizardPage.STRING) ) {
			vContext.put("primaryKey","S");
			vContext.put("pkClass", "java.lang.String");
		} else
		if( s.equals(CreateEJBWizardPage.INTEGER) ) {
			vContext.put("primaryKey","I");
			vContext.put("pkClass", "java.lang.Integer");
		} else
		if( s.equals(CreateEJBWizardPage.OBJECT) ) {
			vContext.put("primaryKey","O");
			vContext.put("pkClass", "java.lang.Object");
		}
		s = EjbPage.getInterfaceName();
		if( s.equals(CreateEJBWizardPage.LOCAL) ) {
			vContext.put("location","L");
            vContext.put("isLocal", TRUE);
		} else
		if( s.equals(CreateEJBWizardPage.REMOTE) ) {
			vContext.put("location","R");
            vContext.put("isLocal", FALSE);
		}


        vContext.put("SESSION_BEAN", SESSION_BEAN);
        vContext.put("ENTITY_BEAN", ENTITY_BEAN);
        vContext.put("MESSAGE_DRIVEN_BEAN", MESSAGE_DRIVEN_BEAN);
        vContext.put("STATELESS", STATELESS);
        vContext.put("STATEFUL", STATEFUL);
        vContext.put("CMP1", CMP1);
        vContext.put("CMP2", CMP2);
        vContext.put("BMP", BMP);

		
		
		vContext.put("jonasroot",jonasRoot);
		vContext.put("jonasbase",JonasLauncherPlugin.getDefault().getBaseDir());
		vContext.put("outputdirectory",project.getOutputFolder());
		vContext.put("outputJAR",project.getProject().getLocation().toOSString());
		vContext.put("projectName",project.getProject().getName());
		
		//On calcule le nombre de fragments du package
		String fragment = "../..";
		for (int i=0;i<pkgName.length();i++) {
			if (pkgName.charAt(i) == ".".charAt(0)) {
				JonasLauncherPlugin.log("On passe dans if");
				fragment = fragment.concat("/..");
			}
		}
		
		vContext.put("relatifProjet",fragment);				
		vContext.put("pkgNameJava",pkgName);

		pkgName = pkgName.replace(".".charAt(0),File.separatorChar);
		
		Vector pkgListe = project.getPackageProjectFolder();

		// Generates webapps files
		try {
			vEngine = new VelocityEngine();
			vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
			vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
			vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
							jonasRoot.concat("/templates/newbean"));

			try {
				vEngine.init();
			} catch (Exception e) {
				fatalError("unable to initilise Velocity engine (" + e + ")");
			}
			
			Integer MESSAGE_DRIVEN_BEAN =
	            (Integer)vContext.get("MESSAGE_DRIVEN_BEAN");
			Integer beanFlavorInVC = (Integer)vContext.get("beanFlavor");

			boolean isClientToGenerate = true;
			vContext.put("beanType","EJB");
			File fileGeneration = new File(project.getProject().getLocation().append(project.getSrcEJBDir()).append(pkgName).toOSString()); 
            generate("ejb-jar.vm", fileGeneration + "/" + jarName + ".xml");
            generate("jonas-ejb-jar.vm", fileGeneration + "/" + "jonas-" + jarName + ".xml");
            if (beanFlavorInVC != MESSAGE_DRIVEN_BEAN) {
                String local = "";
                Boolean isLocal = (Boolean)vContext.get("isLocal");
                if (isLocal.booleanValue()) {
                    local = "Local";
                    isClientToGenerate = false;
                }
                generate("remote.vm", fileGeneration + "/" + beanName + local + ".java");
                generate("home.vm", fileGeneration + "/" + beanName + local + "Home.java");
            }
			vEngine = new VelocityEngine();
			vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
			vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
			vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
					TemplateDirUtil.getTemplateDir().concat("/newbean"));
			try {
				vEngine.init();
			} catch (Exception e) {
				fatalError("unable to initilise Velocity engine (" + e + ")");
			}

			vContext.put("beanType","EJB");
			generate("bean.vm", fileGeneration + "/" + beanName + "EJB" + ".java");
			if (isClientToGenerate) {
				generate("client.vm", fileGeneration + "/" + beanName + "Client.java");
			}
            generate("build.vm", fileGeneration + "/" + "build.xml");
			vContext.put("pkgListe", pkgListe);
			generate("buildgene.vm", project.getProject().getLocation().append(project.getSrcEJBDir()).append("build.xml").toOSString());
            System.out.println("Your bean files have been created. You can now customize them.");

		} catch (Exception e) {        	
			error(e.toString());
		}

		

		// now to refresh project so it picks up the new files
		try{
			project.setPackageProjectFolder(pkgName);
			project.setJarEjbName(EjbPage.getJarName());
			//project.setSrcAsSourceFolder(EjbPage.getSrcDir()); 
			project.getProject().refreshLocal( IResource.DEPTH_INFINITE, null );			 
		}
		catch( CoreException e ) {}

	}

	public void			setNature( JonasProject nature )
	{
		jonasNature = nature;
	}

	public void			setPkgFolder( IFolder f )
	{
		pkgfolder = f;
	}

	/**
	 * Create a new array : [ s1, s2, s3 ]
	 * (TODO move this method to StringUtil)
	 */
	public static String[] concat( String[] s1, String s2, String s3 )
	{
		String[] full = new String[ s1.length + 2 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-2 ] = s2;
		full[ full.length-1 ] = s3;
		return full;	
	}

	/**
	 * Create a new array : [ s1, s2 ]
	 * (TODO move this method to StringUtil)
	 */
	public static String[] concat( String[] s1, String s2 )
	{
		String[] full = new String[ s1.length + 1 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-1 ] = s2;
		return full;	
	}

    /**
     * Generates a file from the specified template.
     * @param templateFileName the name of the template file
     * @param targetFileName the name of the generated file
     */
    private void generate(String templateFileName,
                          String targetFileName) throws Exception, IOException, ResourceNotFoundException, ParseErrorException, MethodInvocationException {
        FileWriter fileWriter = null;
        fileWriter = new FileWriter(targetFileName);
        vEngine.mergeTemplate(templateFileName, vContext, fileWriter);
        fileWriter.close();
    }

	
    /**
     * Display the specified error message.
     * @param errMsg the error message to display
     */
    static void error(String errMsg) {
        System.err.println("NewBean error: " + errMsg);
    }


    /**
     * Display the specified error message and exits with an
     * EXIT_FAILURE status.
     * @param errMsg the error message to display
     */
    static void fatalError(String errMsg) {
        System.err.println("NewBean fatal error: " + errMsg);
        System.exit(EXIT_FAILURE);
    }

}
